/*
* usr_ind.c - contains user indication functions
*
* Copyright (c) 2012 emtas GmbH
*-------------------------------------------------------------------
* $Id: usr_ind.c 4319 2013-08-13 08:53:30Z  $
*
*
*-------------------------------------------------------------------
*
*
*/

/********************************************************************/
/**
* \file
* \brief user indications
*
*/

/* header of standard C - libraries
---------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* header of project specific types
---------------------------------------------------------------------------*/
#include <gen_define.h>
#include <eb_energybus.h>

#ifdef BOOT
# include <fwupdate.h>
#endif

#include "battery.h"


/* constant definitions
---------------------------------------------------------------------------*/

/* local defined data types
---------------------------------------------------------------------------*/

/* list of external used functions, if not in headers
--------------------------------------------------------------------------*/

/* list of global defined functions
---------------------------------------------------------------------------*/

/* list of local defined functions
---------------------------------------------------------------------------*/

/* external variables
---------------------------------------------------------------------------*/

/* global variables
---------------------------------------------------------------------------*/

/* local defined variables
---------------------------------------------------------------------------*/


/***************************************************************************/
/**
* \brief indNmt - NMT indication
*
* \param state
    local nmt state
* \results
*	nothing
*/
void indNmt(
		UNSIGNED8	state
	)
{
	switch (state)  {
		case 4:
			printf("EB NMT ind STOPPED\n");
			break;
		case 5:
			printf("EB NMT ind OPERATIONAL\n");
			break;
		case 127:
			printf("EB NMT ind PRE-OPERATIONAL\n");
			break;
		case 128:
			printf("EB NMT ind Reset Node\n");
			break;
		case 129:
			printf("EB NMT ind RESET COMM\n");
			break;
		default:
			printf("EB NMT ind %d\n", state);
	}
}


/***************************************************************************/
/**
* \brief indEbcMon - EbcMon indication
*
* \param
*	ebc=1, ext charger=0
*	ok = 1, not ok = 0
* \results
*	nothing
*/
void indEbcMon(
		UNSIGNED8 ebc,
		UNSIGNED8 active
	)
{
	if (ebc == 1) {
		if (active == 1)  {
			printf("EB ind ebc: on\n");
		} else {
			printf("EB ind ebc: off\n");
		}
	} else {
		if (ebc == 127)  {
			if (active == 1)  {
				printf("EB ind externer Charger: on\n");
			} else {
				printf("EB ind externer Charger: off\n");
			}
		}
	}
}


/***************************************************************************/
/**
* \brief indState - eb state changed
*
* \param
*	new state
* \results
*	nothing
*/
void indEbState(
		UNSIGNED16	state,
		UNSIGNED8 vdnNr
	)
{
	switch (state)  {
		case EB_STATE_DISCONNECTED:
			printf("EB state changed VDN %d: DISCONNECTED\n", vdnNr);
			break;
		case EB_STATE_CONNECTED:
			printf("EB state changed VDN %d: CONNECTED\n", vdnNr);
			break;
		case EB_STATE_COMP_CHECK:
			printf("EB state changed VDN %d: COMP-CHECK\n", vdnNr);
			break;
		case EB_STATE_LIMITING:
			printf("EB state changed VDN %d: LIMITING\n", vdnNr);
			break;
		case EB_STATE_OPERATING:
			printf("EB state changed VDN %d: OPERATING\n", vdnNr);
			break;
		case EB_STATE_MASTERLESS:
			printf("EB state changed VDN %d: MASTERLESS\n", vdnNr);
			break;
		case EB_STATE_SLEEP:
			printf("EB state changed VDN %d: SLEEP\n", vdnNr);
			break;
		default:
			break;
	}
}


/***************************************************************************/
/**
* \brief indEbCheckState - check new eb state
*
* \param
*	new state
* \results
*	state change allowed
*/
BOOL_T indEbCheckState(
		UNSIGNED16	state,
		UNSIGNED8 vdnNr		/**< virtual device number */
	)
{
BOOL_T	stateAllowed = CO_TRUE;

	switch (state)  {
		case EB_STATE_DISCONNECTED:
			printf("EB check state VDN %d to DISCONNECTED allowed\n", vdnNr);
			break;
		case EB_STATE_CONNECTED:
			printf("EB check state VDN %d to CONNECTED allowed\n", vdnNr);
			break;
		case EB_STATE_COMP_CHECK:
			printf("EB check state VDN %d to COMP-CHECK: allowed.\n", vdnNr);
			break;
		case EB_STATE_LIMITING:
			printf("EB check state VDN %d to LIMITING: allowed.\n", vdnNr);
			break;
		case EB_STATE_OPERATING:
			printf("EB check state VDN %d to OPERATING: allowed.\n", vdnNr);
			break;
		case EB_STATE_MASTERLESS:
			printf("EB check state VDN %d to MASTERLESS: not allowed.\n", vdnNr);
			stateAllowed = CO_FALSE;
			break;
		default:
			break;
	}
	return(stateAllowed);
}


/***************************************************************************/
/**
* \brief indBatState - new battery state
*
* \param
*	new state
* \results
*	none
*/
void indBatState(
		UNSIGNED16	state,
		UNSIGNED8 vdn
	)
{
	(void)vdn;

	switch (state)  {
		case EB_BAT_STATE_DO_NOT_ATTACH:
			printf("EB BATTERY VDN %d state Do Not Attach\n", vdn);
			break;
		case EB_BAT_STATE_READY_TO_ATTACH:
			printf("EB BATTERY VDN %d state Ready To Attach\n", vdn);
			break;
		case EB_BAT_STATE_ERROR:
			printf("EB BATTERY VDN %d state Error\n", vdn);
			break;
		case EB_BAT_STATE_PLEASE_DETACH:
			printf("EB BATTERY VDN %d state Please Detach\n", vdn);
			break;
		case EB_BAT_STATE_NORMAL_OPERATION:
			printf("EB BATTERY VDN %d state Normal Operation\n", vdn);
			break;
		default:
			break;
	}
}
/***************************************************************************/
/**
* \brief indBatCheckState - check new battery state
*
* \param
*	new state
* \results
*	state change allowed
*/
BOOL_T indBatCheckState(
		UNSIGNED16	state,
		UNSIGNED8 vdn
	)
{
BOOL_T	stateAllowed = CO_TRUE;

	(void)vdn;

	switch (state)  {
		case EB_BAT_STATE_DO_NOT_ATTACH:
			printf("EB BATTERY VDN %d state Do Not Attach: allowed\n", vdn);
			break;
		case EB_BAT_STATE_READY_TO_ATTACH:
			printf("EB BATTERY VDN %d state Ready To Attach: allowed\n", vdn);
			break;
		case EB_BAT_STATE_ERROR:
			printf("EB BATTERY VDN %d state Error: allowed\n", vdn);
			break;
		case EB_BAT_STATE_PLEASE_DETACH:
			printf("EB BATTERY VDN %d state Please Detach: allowed\n", vdn);
			break;
		case EB_BAT_STATE_NORMAL_OPERATION:
			printf("EB BATTERY VDN %d state Normal Operation: allowed\n", vdn);
			break;
	}

	return(stateAllowed);
}


/***************************************************************************/
/**
* \brief indSdoWrite - sdo write ind
*
* \param
*	new state
* \results
*	state change allowed
*/
void indSdoWrite(
		UNSIGNED16	index,
		UNSIGNED8	subIndex
	)
{
#ifdef BOOT
UNSIGNED8 u8tmp;
RET_T retVal;
#endif

	printf("EB SDO WRITE %x:%d\n", index, subIndex);

#ifdef BOOT
	/*
	 * call bootloader after 'stop program' command
	 * write '0' to 0x1F51:1
	 * additional you can check passwords and so on
	 */
	if((index == 0x1F51) && (subIndex == 1)) {
		retVal = coOdGetObj_u8(0x1F51, 1, &u8tmp);
		if (retVal != RET_OK) {
			return;
		}

		if (u8tmp != 0 /* Stop program */) {
			return;
		}

	 	jump2Bootloader();
	}
#endif
}



/***************************************************************************/
/**
* \brief indCanState
*
* \param event
*	CAN Bus event
*/
void indCanState(
		CO_CAN_STATE_T event
	)
{
	switch (event) {
		case CO_CAN_STATE_BUS_OFF:
			printf("CAN State: Busoff.\n");
			break;
		case CO_CAN_STATE_BUS_ON:
			printf("CAN State: Bus on (Error Active).\n");
			break;
		case CO_CAN_STATE_PASSIVE:
			printf("CAN State: Error Passive.\n");
			break;
		case CO_CAN_STATE_UNCHANGED:
			printf("CAN State unchanged.\n");
			break;
		default:
			printf("CAN State 0x%x\n", event);
			break;
	}
}


/***************************************************************************/
/**
* \brief indErrorHandler - Error ind
*
* \param event
*	error event
*/
void indErrorHandler(
		CO_COMM_STATE_EVENT_T event
	)
{
	printf("EB Error Event 0x%x --> ", event);

	switch (event) {
		case CO_COMM_STATE_EVENT_NONE:
			printf("(no event)\n");
			break;

		case CO_COMM_STATE_EVENT_CAN_OVERRUN:
			printf("EB error handler: CAN Overrun.\n");
			break;
		case CO_COMM_STATE_EVENT_REC_QUEUE_FULL:
			printf("EB error handler: Recv Queue full.\n");
			break;
		case CO_COMM_STATE_EVENT_REC_QUEUE_OVERFLOW:
			printf("EB error handler: Recv Queue overflow.\n");
			break;
		case CO_COMM_STATE_EVENT_REC_QUEUE_EMPTY:
			printf("EB error handler: Recv Queue empty.\n");
			break;
		case CO_COMM_STATE_EVENT_TR_QUEUE_FULL:
			printf("EB error handler: Tx Queue full.\n");
			break;
		case CO_COMM_STATE_EVENT_TR_QUEUE_OVERFLOW:
			printf("EB error handler: Tx Queue overflow.\n");
			break;
		case CO_COMM_STATE_EVENT_TR_QUEUE_EMPTY:
			printf("EB error handler: Tx Queue empty.\n");
			break;
		default:
			printf("EB error handler: unknown.\n");
	}
}


#ifdef EB_SECURITY_CNT
/***************************************************************************/
/**
* \brief indSecurity - security event
*
* \param
*	new event
* \results
*	event was executed
*/
BOOL_T indSecurity(
		EB_SEC_EVENT_T	event,
		UNSIGNED16		devType,
		UNSIGNED8		vdn
	)
{
printf("indSecurity event %x\n", event);
	switch (event)  {
		case EB_SEC_EVENT_LOCK:
printf("******* BATTERY LOCKED *****\n");
			/* if it takes some time, call the function later */
			ebSecLocked(vdn, CO_TRUE);

			break;

		case EB_SEC_EVENT_UNLOCK:
printf("******* BATTERY UNLOCKED *****\n");
			/* if it takes some time, call the function later */
			ebSecLocked(vdn, CO_FALSE);

			break;

		default:
			break;
	}

	return(CO_TRUE);
}
#endif /* EB_SECURITY_CNT */


/***************************************************************************/
/**
* \brief
*
* \internal
*
*/
EB_RET_T eepromWriteInd(
		EB_EEPROM_DATA_T type,
		UNSIGNED16 number,
		UNSIGNED16 len,
		CO_CONST UNSIGNED8 *src
	)
{
#ifdef TARGET_LINUX
FILE	*fd;
char	*fname;
UNSIGNED16	offs;

	switch (type)  {
		case EB_EEPROM_KNOWN_DEVICES:
			fname = "eeprom_knowndevices.bin";
			offs = len * number;
			break;
		case EB_EEPROM_SECURITY_KEYS:
			fname = "eeprom_sec_keys.bin";
			offs = len * number;
			break;
		default:
			return(EB_RET_ERROR);
	}

printf("** eepromWriteInd: load %s, num: %d offs: %x, len: %d\n", fname, number, offs, len);

	fd = fopen(fname, "ab");
	if (fd == NULL)  {
		return(EB_RET_ERROR);
	}

	fseek(fd, offs, SEEK_SET);
	fwrite(src, 1, len, fd);
	fclose(fd);
#endif /* TARGET_LINUX */

	return(EB_RET_OK);
}


/***************************************************************************/
/**
* \brief
*
* \internal
*
*/
EB_RET_T eepromReadInd(
		EB_EEPROM_DATA_T type,
		UNSIGNED16 number,
		UNSIGNED16 len,
		UNSIGNED8 *dst
	)
{
#ifdef TARGET_LINUX
FILE	*fd;
char	*fname;
UNSIGNED16	offs;

	switch (type)  {
		case EB_EEPROM_KNOWN_DEVICES:
			fname = "eeprom_knowndevices.bin";
			offs = len * number;
			break;
		case EB_EEPROM_SECURITY_KEYS:
			fname = "eeprom_sec_keys.bin";
			offs = len * number;
			break;
		default:
			return(EB_RET_ERROR);
	}

printf("** eepromReadInd: load %s, num: %d offs: %x, len: %d\n", fname, number, offs, len);

	fd = fopen(fname, "rb");
	if (fd == NULL)  {
		return(EB_RET_ERROR);
	}

	fseek(fd, offs, SEEK_SET);
	fread(dst, 1, len, fd);
	fclose(fd);
#endif /* TARGET_LINUX */

	return(EB_RET_OK);
}




/***************************************************************************/
/**
* \brief iebSleepInd
*
* \internal
*
*/
UNSIGNED8 sleepInd(
		CO_SLEEP_MODE_T	mode
	)
{
	printf("sleepInd: ");

	switch (mode)  {
		case CO_SLEEP_MODE_CHECK:
			printf("sleep mode check\n");
			break;
		case CO_SLEEP_MODE_PREPARE:
			break;
		case CO_SLEEP_MODE_SILENT:
			printf("start silent mode\n");
			break;
		case CO_SLEEP_MODE_DOZE:
			printf("doze\n");

printf("******************************* SLEEP\n");
/* disable CAN */
/* disable timer */
#if 0
/* works only for can4linux, not for socketCan */
{
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include "/usr/src/can4linux/can4linux.h"

int canfd;
canmsg_t msg;
int ret = -1;

	/* open std can device */
	canfd = open(C4L_DEVICE, O_RDONLY);
	/* wait until next message */
	while (ret < 0)  {
		ret = read(canfd, &msg, 1);
	}
	close(canfd);
}
#endif
			/* start wakeup */
printf("******************************* WAKEUP\n");
			/* init CAN */

			coSleepAwake(); /* wakeup command */

			break;
		default:
			printf("unknown\n");
	}

	return(0);
}
